/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 6.2.2018
 * Time: 17:06
 * Author Petar Vanchev (petarv@qnext.com)
 * Author Milen Dimitrov (milend@qnext.com)
 */

var CompEncryption = function ($container) {
    BaseComp.call(this, $container);

    this._initContent();
};

QnUtil.inherits(CompEncryption, BaseComp);
QnUtil.extend(CompEncryption.prototype, {

    /* Public method */

    updatePanel: function () {
        // enable disable SGX logo next to radio buttons
        var dataguard = !!getGlobal(KEY_GLOBAL_DATAGUARD);
        var $encComp = $(".method-encrypt-group");
        $encComp.toggleClass("methodEncryptOn", dataguard)
            .toggleClass("methodEncryptOff", !dataguard);

        var encrypt = !!getGlobal(KEY_GLOBAL_ENCRYPT);
        var showEncChoice = !!getGlobal(KEY_GLOBAL_VIEW_ONLY) && encrypt;
        var listEnc = getGlobal(KEY_GLOBAL_LIST_METHOD_ENCRYPT_ALLOW);

        if (showEncChoice && listEnc.length === 1 && listEnc[0] === ENC_NONE)
            showEncChoice = false;

        if (showEncChoice) {
            this._switchMode(false);
            this.updateEncComps();

            // parse security and set checked radio buttons and checkbox
            var security = this._parseSecurity();
            $("input[name=method_encrypt][value=" + security["methodEncrypt"] + "]").trigger("click");
            $("#media_encrypt").prop("checked", security["mediaEncrypt"]);

            // FF-2354: Add tooltip in encryption components in passive's per-user settings
            if (encrypt) {
                setTextContext("field_method_encrypt_info");
                // add focus handler
                $encComp.on("focus", "input", function () {
                    switchTT($encComp.parent());
                });
                // add hover handler
                $encComp.hover(function () {
                    switchTT($encComp.parent());
                }, function () {
                    $("body").find(".tooltipCont").css("display", "none");
                    $(':focus').focus();
                });
            }
        } else {
            // FF-2208: Show no-encryption-text - add text if if only no encryption is supported
            // also show that text in case when viewOnly = false see https://jira2.cnexus.com/browse/FF-2247
            this._switchMode(true);
        }
    },

    /**
     * Update radio buttons and media encrypt checkbox
     *
     * @param {Boolean} [onDisable] When true will disable all controls (radio buttons and checkbox). Default is false.
     */
    updateEncComps: function (onDisable) {
        var listEnc = getGlobal(KEY_GLOBAL_LIST_METHOD_ENCRYPT_ALLOW);
        this._enableEncComp(ENC_NONE, !onDisable && listEnc ? listEnc.indexOf(ENC_NONE) !== -1 : false);
        this._enableEncComp(ENC_RAW, !onDisable && listEnc && getGlobal(KEY_GLOBAL_ENCRYPT) ? listEnc.indexOf(ENC_RAW) !== -1 : false);
        this._enableEncComp(ENC_SGX, !onDisable && listEnc && getGlobal(KEY_GLOBAL_DATAGUARD) ? listEnc.indexOf(ENC_SGX) !== -1 : false);

        this._updateMediaEnc();
    },

    /**
     * Checks whether all components are hidden because there is no sense to select any.
     *
     * @return {boolean} true if this component is Hidden
     * @readonly
     */
    isHidden: function () {
        // Check whether encryption controls are hidden - then
        return $('.method-encrypt-group').hasClass("noEncryption");
    },

    /**
     * Get current encryption method selected.
     *
     * @return {string} one of {@link ENC_NONE}, {@link ENC_RAW} or {@link ENC_SGX}.
     * @readonly
     */
    getMethodEncrypt: function () {
        return this._getCheckedEncComp();
    },

    /**
     * !!! Do not override
     * @return {Number} security bits
     * @readonly
     */
    getSecurity: function () {
        var methodEncrypt = this._getCheckedEncComp();
        var mediaEncrypt = this._isMediaEncrypt();
        var security = 0;
        switch (methodEncrypt) {
            case ENC_NONE:
                break;
            case ENC_RAW:
                security = changeBits(security, CompEncryption._SEC_BIT_ENCRYPT, true);
                break;
            case ENC_SGX:
                security = changeBits(security, CompEncryption._SEC_BITS_SGX, true);
                break;
            default:
                log("Unknown encrypt method: " + methodEncrypt);
                methodEncrypt = ENC_NONE;
                break;
        }
        if (methodEncrypt !== ENC_NONE && !mediaEncrypt) {
            security = changeBits(security, CompEncryption._SEC_BITS_NO_ENCRYPT_MEDIA, true);
        }
        log("Use security: 0b" + security.toString(2) + " (" + security + ")");

        return security;
    },

    /**
     * Check did client and passive encryption are compatible (client can add/edit passive with such encryption)
     *
     * @param {Boolean} [forActivate]
     * @return {Boolean} return true if list with current client allowed encryption level contains
     *                   one reported from passive in status
     */
    checkEncryption: function (forActivate) {
        var clientListEnc = getGlobal(KEY_GLOBAL_LIST_METHOD_ENCRYPT_ALLOW);
        var isSupported = false;
        if (forActivate) {
            var encrypt = getGlobal(KEY_GLOBAL_ENCRYPT);
            var dataguard = getGlobal(KEY_GLOBAL_DATAGUARD);

            if (encrypt && !dataguard) {
                isSupported = clientListEnc.indexOf(ENC_NONE) !== -1 || clientListEnc.indexOf(ENC_RAW) !== -1;
            } else if (encrypt && dataguard) {
                isSupported = clientListEnc.indexOf(ENC_NONE) !== -1 || clientListEnc.indexOf(ENC_RAW) !== -1 ||
                    clientListEnc.indexOf(ENC_SGX) !== -1;
            } else {
                isSupported = clientListEnc.indexOf(ENC_NONE) !== -1;
            }
        } else {
            var sec = this._parseSecurity();
            var psvMethodEnc = sec["methodEncrypt"];
            isSupported = clientListEnc.indexOf(psvMethodEnc) > -1;
        }

        if (isSupported) {
            // clear unsupported error message
            clearMessage(ERROR_UNSUPPORTED_ENCRYPTION);
        } else {
            // show unsupported error message
            showMessage(ERROR_UNSUPPORTED_ENCRYPTION, "error_invalid_encryption_configuration");
        }

        return isSupported;
    },


    /* Protected method */

    /**
     *
     * @protected
     */
    _initContent: function () {
        var $cont = $('<div class="method-encrypt-group form-group"><div class="control-label"></div></div>');
        $cont.find(".control-label").setTextById("field_method_encrypt_header");
        this._renderRadioEnc($cont, "method_encrypt_none", ENC_NONE);
        this._renderRadioEnc($cont, "method_encrypt_raw", ENC_RAW);
        this._renderRadioEnc($cont, "method_encrypt_sgx", ENC_SGX);

        $('<div class="media-encrypt-group">' +
            '<input type="checkbox" id="media_encrypt" class="field_media_encrypt_input" name="media_encrypt">' +
            '<span class="field_method_encrypt_label"></span><br/>' +
            '</div>')
            .appendTo($cont)
            .find("span")
            .setTextById("field_media_encrypt");

        $cont.appendTo(this._getCont());

        // add tooltip
        this._renderTooltip();

        var methodEncMin = this._getMethodEncMin();
        // Select by default lowest possible
        $("input[name=method_encrypt][value=" + methodEncMin + "]").prop("checked", true);

        var self = this;
        $("input[name=method_encrypt]").change(function () {
            self._updateMediaEnc();
        });

        this.updateEncComps();
        // init in noEncryption mode (encryption options checkbox will be hidden)
        this._switchMode(true);
    },


    /* Private method */

    /**
     * Renders/adds new input+label (span) with given "id" and
     * given value. String UID is created using "field_"+[id].
     *
     * @param {jQuery} $cont container where to add new input element
     * @param {string} id component ID to be set as "id" attribute.
     * Used to create string too.
     * @param {string} value value to be set to input
     * @private
     */
    _renderRadioEnc: function ($cont, id, value) {
        $('<input type="radio" name="method_encrypt" id="' + id + '" ' +
            'class="field_method_encrypt_input" value="' + value + '">')
            .appendTo($cont);
        $('<span class="field_method_encrypt_label"></span>')
            .setTextById("field_" + id).appendTo($cont);
        $cont.append("<br/>");
    },
    /**
     * Add general tooltip for encryption comp that will be visible if any component is on focus or is hovered
     *
     * @private
     */
    _renderTooltip: function () {
        var $cont = this._getCont().find(".method-encrypt-group");
        this._addTooltip($cont, "MethodEncrypt", "field_method_encrypt");
    },

    /**
     * Parse security bits
     *
     * @return {Object} always valid. Default is:
     * {
     *      "methodEncrypt" : "None",
     *      "mediaEncrypt": false
     * }
     * @Private
     */
    _parseSecurity: function () {
        var security = {
            "methodEncrypt": ENC_NONE,
            "mediaEncrypt": false
        };


        var securityBit = getGlobal(KEY_GLOBAL_DEVICE_SECURITY);
        if (securityBit >= 0) {
            log("Parse Security from status.");

            // parse encryption bits
            var methodEncrypt = ENC_NONE;
            var hasEncrypt = isBitsSet(securityBit, CompEncryption._SEC_BIT_ENCRYPT);
            var hasPassiveAtt = isBitsSet(securityBit, CompEncryption._SEC_BIT_PASSIVE_ATT);
            var hasClientAtt = isBitsSet(securityBit, CompEncryption._SEC_BIT_CLIENT_ATT);
            if (true === hasPassiveAtt || true === hasClientAtt) {
                if (false === hasEncrypt || hasPassiveAtt !== hasClientAtt)
                    log("Wrong encrypt int " + securityBit + " (" + securityBit.toString(2) + "), will fix to" + ENC_SGX, true);

                methodEncrypt = ENC_SGX;
            } else if (hasEncrypt) {
                methodEncrypt = ENC_RAW;
            } else {
                methodEncrypt = ENC_NONE;
            }

            security["methodEncrypt"] = methodEncrypt;

            // parse media encrypt bits
            var mediaEncrypt = false;
            var hasNoVideoEnc = isBitsSet(securityBit, CompEncryption._SEC_BIT_NO_ENCRYPT_VIDEO);
            var hasNoAudioEnc = isBitsSet(securityBit, CompEncryption._SEC_BIT_NO_ENCRYPT_AUDIO);
            if (methodEncrypt !== ENC_NONE) {
                if (hasNoVideoEnc || hasNoAudioEnc) {
                    if (hasNoVideoEnc !== hasNoAudioEnc)
                        log("Wrong media encrypt int " + securityBit + " (" + securityBit.toString(2) + "), will fix to no media encrypt", true);
                } else {
                    mediaEncrypt = true;
                }
            }

            security["mediaEncrypt"] = mediaEncrypt;
        } else {
            log("Security not received from status. Will use default: " + JSON.stringify(security));
        }

        return security;
    },

    /**
     * Show/hide encryption radio buttons
     *
     * @param {Boolean} isNoEncrypt
     * @private
     */
    _switchMode: function (isNoEncrypt) {
        // hide encryption controls
        $('.method-encrypt-group').toggleClass("noEncryption", isNoEncrypt);
    },

    /**
     *
     * @param {String} methodEnc
     * @param {Boolean} [enabled]
     * @private
     */
    _enableEncComp: function (methodEnc, enabled) {
        $("input[name=method_encrypt][value=" + methodEnc + "]").prop("disabled", !enabled);
    },

    /**
     *  Get minimum supported encryption reported from server
     *
     * @return {String}
     * @private
     */
    _getMethodEncMin: function () {
        var listEnc = getGlobal(KEY_GLOBAL_LIST_METHOD_ENCRYPT_ALLOW);
        var methodEncMin;
        if (!listEnc || listEnc.length <= 0) {
            methodEncMin = ENC_NONE;
        } else {
            // check for NONE
            if (listEnc.indexOf(ENC_NONE) !== -1)
                methodEncMin = ENC_NONE;
            // check for RAW
            if (!methodEncMin && listEnc.indexOf(ENC_RAW) !== -1)
                methodEncMin = ENC_RAW;
            // check for SGX
            if (!methodEncMin && listEnc.indexOf(ENC_SGX) !== -1)
                methodEncMin = ENC_SGX
        }

        return methodEncMin;
    },

    /**
     * @return {String}
     * @private
     */
    _getCheckedEncComp: function () {
        return $('input[name=method_encrypt]:checked').val();
    },

    /**
     *
     * @return {Boolean}
     * @private
     */
    _isMediaEncrypt: function () {
        return $("#media_encrypt").prop("checked");
    },

    /**
     * Update media encrypt checkbox state
     *
     * @private
     */
    _updateMediaEnc: function () {
        var methodCur = this._getCheckedEncComp();
        var isAllDisabled = true;
        $('input[name=method_encrypt]').each(function () {
            if (!$(this).prop("disabled")) {
                isAllDisabled = false;
                return false;
            }
        });

        var $media = $("#media_encrypt");
        $media.prop("disabled", isAllDisabled || methodCur === ENC_NONE);
        if (methodCur === ENC_NONE)
            $media.prop("checked", false);
    }
});

/**
 * Raw encryption (no SGX/attestation)
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BIT_ENCRYPT = 1;             // 0b00000001
/**
 * Passive attestation required
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BIT_PASSIVE_ATT = 2;         // 0b00000010
/**
 * Client attestation required
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BIT_CLIENT_ATT = 4;          // 0b00000100
/**
 * SGX encryption - both client and passive attestation required
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BITS_SGX = CompEncryption._SEC_BIT_ENCRYPT | CompEncryption._SEC_BIT_PASSIVE_ATT |
    CompEncryption._SEC_BIT_CLIENT_ATT;          // 0b00000111 - SEC_BIT_ENCRYPT | SEC_BIT_PASSIVE_ATT | SEC_BIT_CLIENT_ATT;
/**
 * Video data unencrypted
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BIT_NO_ENCRYPT_VIDEO = 8;   // 0b00001000
// /**
//  * Images download unencrypted
//  *
//  * @type {number}
//  * @protected
//  * @constant
//  */
// CompEncryption._SEC_BIT_NO_ENCRYPT_IMAGE = 16;  // 0b00010000
/**
 * Audio/music data unencrypted
 *
 * @type {number}
 * @protected
 * @constant
 */
CompEncryption._SEC_BIT_NO_ENCRYPT_AUDIO = 32;  // 0b00100000
/**
 * Both Audio and Video data unencrypted
 *
 * @type {number}
 * @private
 * @constant
 */
CompEncryption._SEC_BITS_NO_ENCRYPT_MEDIA = CompEncryption._SEC_BIT_NO_ENCRYPT_VIDEO |
    CompEncryption._SEC_BIT_NO_ENCRYPT_AUDIO;   // 0b00101000