/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 15.3.2018
 * Time: 10:56
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 * Frame that contain encryption choice component {@link CompEncryption}
 *
 * @param {String} vid
 * @param {jQuery} $container when not provided frame will be added to the body element
 * @param {Object} [options] see {@link FrameMain} doc
 * @constructor
 */
var FrameEncrypt = function (vid, $container, options) {
    /**
     *
     * @type {CompEncryption} can be null
     * @protected
     */
    this._compEncryption = null;

    FrameWithForm.call(this, vid, $container, options);
};

QnUtil.inherits(FrameEncrypt, FrameWithForm);
QnUtil.extend(FrameEncrypt.prototype, {

    /* Public method */

    updateFrame: function () {
        if(this._hasEncComp())
            this._compEncryption.updatePanel();
        // special cases requirements see http://jira.cnexus.com/browse/FFS-4522
        if (getGlobal(KEY_GLOBAL_DEVICE_OS) === "linux" && getGlobal(KEY_GLOBAL_DEVICE_SCHEME) === "tplink") {
            var $usrField = this._getFormControl(FrameWithForm.FIELD_USERNAME);
            if($usrField && $usrField.length >= 1) {
                $usrField.attr('readonly', true).val("root"); // set username field as readonly and value to "root"
            }
            var $domain = this._getFormControl(FrameWithForm.FIELD_DOMAIN);
            if($domain && $domain.length >= 1) {
                $domain.attr('readonly', true); // make filed read only

                // set the domain if passive provided one (set in the admin panel)
                var domain = getGlobal(KEY_GLOBAL_DOMAIN);
                // if(domain && domain.length >=1)
                $domain.val(domain);
            }
        }

        // set tooltip text and images to activate and update page
        setTextContext("field_username_info");
        setTextContext("field_password_info");
        setTextContext("field_domain_info");
        setImageContext("scrDomain");
        setImageContext("scrUsername");
        setImageContext("scrPassword");

        FrameEncrypt.superclass.updateFrame.apply(this);
    },


    /* Protected method */

    /**
     * Check did frame has valid instance of {@link CompEncryption}
     *
     * @return {Boolean} always valid
     * @protected
     */
    _hasEncComp: function () {
        return !!this._compEncryption;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getFieldIdList: function () {
        return [FrameWithForm.FIELD_USERNAME, FrameWithForm.FIELD_PASSWORD, FrameWithForm.FIELD_DOMAIN];
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getControlIdPrefix: function () {
        return "user-";
    },


    /**
     *
     * @inheritDoc
     * @protected
     */
    _isRequired: function (fieldId) {
        switch (fieldId) {
            case FrameWithForm.FIELD_PASSWORD:
            case FrameWithForm.FIELD_DOMAIN:
            case FrameWithForm.FIELD_CODE:
                return false;
            default:
                return true;
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getFieldParams: function (fieldName) {
        var params = {
            "inputType": "text",
            "isRequired": this._isRequired(fieldName),
            "isReadOnly": this._isReadOnly(fieldName),
            "isChecked": this._isChecked(fieldName)
        };

        switch (fieldName) {
            case FrameWithForm.FIELD_USERNAME:
                params["ttContClass"] = "User";
                params["ttPointerClass"] = "Username";
                break;
            case FrameWithForm.FIELD_PASSWORD:
            case FrameWithForm.FIELD_DOMAIN:
            case FrameWithForm.FIELD_CODE:
                if(fieldName === FrameWithForm.FIELD_PASSWORD)
                    params["inputType"] = "password";
                params["ttContClass"] = params["ttPointerClass"] = fieldName.capitalize();
                break;
            default:
                return null;
        }

        return params;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderFormImpl: function ($container) {
        // add encryption comp
        var $encCompCont = this.createCont().addClass("encCompCont").appendTo($container);
        var provCaps = getGlobal(KEY_GLOBAL_PROV_CAPS);
        if(provCaps.indexOf("e") >= 0)
            this._compEncryption = new CompEncryption($encCompCont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        FrameEncrypt.superclass._setControlsState.call(this, onDisable, notForSubmit);
        // update encryption comp
        if(this._hasEncComp())
            this._compEncryption.updateEncComps(onDisable);
        // update input
        this._getFormControl(FrameWithForm.FIELD_USERNAME, true).attr("disabled", onDisable);
        this._getFormControl(FrameWithForm.FIELD_PASSWORD, true).attr("disabled", onDisable);
        this._getFormControl(FrameWithForm.FIELD_DOMAIN, true).attr("disabled", onDisable);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        if(this._hasEncComp()) {
            params["security"] = this._compEncryption.getSecurity();
        } else {
            var globalSec = getGlobal(KEY_GLOBAL_DEVICE_SECURITY);
            if(globalSec >= 0)
                params["security"] = globalSec;
        }
    },

    /**
     * Override to add common implementation for FF-6200/FF-6336
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var toSubmit = true;
        // https://jira2.cnexus.com/browse/FF-6200
        // If there is encryption, show
        // Code was in FrameActivate, but moved here because of FF-6336
        if(this._compEncryption && !this._compEncryption.isHidden()) {
            var method = this._compEncryption.getMethodEncrypt();
            var str = QnLocale.getStr("field_method_encrypt_" + method.toLowerCase());
            toSubmit = confirm(QnLocale.getStr("confirm_enc_mode",
                {listArg: {"encMode": str}, noEsc: true}));
        }
        return toSubmit;
    }



    /* Private method */

});
