/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 6.2.2018
 * Time: 16:20
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {String} vid
 * @param {jQuery} $container
 * @param {Object} [options]
 * Contains:
 *      {Object} [statusOpt]         Options that will be passed to status info command
 *      {String} [statusUrl]         Relative url used when request status info from passive. Not set by default.
 *      {Boolean} [hasHeader]        Default depend of did {@link KEY_GLOBAL_CAPS} contains NoTopToolbar cap
 *      {Boolean} [hasUpdateMsg]     Default is True
 *      {Boolean} [hasHelp]          Default is True
 *      {Boolean} [canSendLogs]      Default is False
 *      {Boolean} [hasHashListener]  Default is False
 *      {Object} [parent]            Parent view
 * @param {Boolean} [doNotRequestStatus]
 *
 * @constructor
 */
var FrameMain = function (vid, $container, options, doNotRequestStatus) {
    BaseComp.call(this, $container);

    var hasHeader = true;
    var caps = getGlobal(KEY_GLOBAL_CAPS);
    if (caps && caps.indexOf("NoTopToolbar") >= 0) {
        hasHeader = false;
    }

    var optDef = {
        "statusOpt": null,
        "statusUrl": null,
        "hasHeader": hasHeader,
        "hasUpdateMsg": true,
        "hasHelp": true,
        "canSendLogs": false,
        "hasHashListener": false
    };

    /**
     *
     * @type {Object}
     * @protected
     */
    this._options = jQuery.extend(optDef, options);
    /**
     * @private
     */
    this._vid = vid;
    /**
     *
     * @type {jQuery}
     * @private
     */
    this._$mainContentCont = null;

    /**
     *
     * @type {Boolean}
     * @private
     */
    this._doNotRequestStatus = !!doNotRequestStatus;

    this._initContent();
};

QnUtil.inherits(FrameMain, BaseComp);
QnUtil.extend(FrameMain.prototype, {

    /* Public methods */

    /**
     * !!! Do not override !!!
     * Request passive status
     *
     * @readonly
     */
    getStatus: function () {
        var self = this;
        psvFetchStatus(function () {
            self.updateFrame(true);
        }, this._options["statusOpt"], this._options["statusUrl"]);
    },

    /**
     *
     * @return {String} always valid
     */
    getVid: function () {
        return this._vid;
    },

    /**
     *
     * @param {Boolean} [isFromStatus] will be true when method is called from {@link #getStatus}
     */
    updateFrame: function (isFromStatus) {
    },

    /* Protected methods */

    /**
     * !!! Do not override use {@link #_initContentImpl} instead
     * @protected
     * @readonly
     */
    _initContent: function () {
        if (this._options["hasHeader"])
            this._renderHeader();
        // Create main content container
        this._$mainContentCont = this.createCont().addClass("contentWrapper " + this.getVid()).appendTo(this._getCont());
        this._initContentImpl(this._$mainContentCont);

        // render updater msg
        if (this._options["hasUpdateMsg"]) {
            this._$mainContentCont.prepend('<div id="updatemsg" class="updateMsg"></div>');
        }

        if (this._options["hasHashListener"]) {
            var self = this;
            $(window).on("hashchange", function () {
                self._initialize();
            });
        }

        this._initialize();
    },

    /**
     * Get main fragment cont
     *
     * @return {jQuery}
     * @protected
     */
    _getMainCont: function () {
        return this._$mainContentCont;
    },

    /**
     *
     * @param {jQuery} $cont
     * @protected
     */
    _initContentImpl: function ($cont) {
        throw new Error("Must be implement _initContentImpl");
    },


    /**
     *
     * @param {jQuery} $cont
     * @protected
     */
    _renderHelpSection: function ($cont) {
        if (this._options["hasHelp"] || this._options["canSendLogs"]) {

            var $comp = $('<div class="desc">' +
                '<div class="helpCont secureonly">' +
                '</div>' +
                '</div>').appendTo($cont);

            if (this._options["hasHelp"]) {
                var $help = $('<p></p>').setTextById("bottom_help_link");
                $comp.find(".helpCont").append($help);
            }

            if (this._options["canSendLogs"]) {
                var $sendLog = $('<a href="#"></a>').addClass("sendLogsLink").setTextById("link_send_logs");
                $comp.find(".helpCont").append($sendLog);

                var self = this;
                $sendLog.on("click", function (e) {
                    self._handleAction("sendLogs", e);
                });
            }
        }
    },

    /* Private methods */

    /**
     * Update send logs link state (Enable/Disable)
     * See https://jira2.cnexus.com/browse/FF-6511
     *
     * @param {Boolean} [isEnabled]
     * @private
     */
    _updateSendLogLinkState: function (isEnabled) {
        if (this._options["canSendLogs"])
            this._getMainCont().find(".sendLogsLink").toggleClass("disabled", !isEnabled);
    },

    /**
     * Render main header
     *
     * @private
     */
    _renderHeader: function () {
        var $comp = $('<div class="headerCont">' +
            '<div class="logoApp"></div>' +
            '<div id="encryptHeaderSgx">' +
            '</div>' +
            '</div>').appendTo(this._getCont());

        var textEncHeader = QnLocale.getStr("text_encrypt_header_1", {"isSpec": true});
        var headerOpts = {
            "listArg": {
                "textEncHeader": textEncHeader
            }
        };
        $comp.find("#encryptHeaderSgx").setTextById("text_encrypt_header", headerOpts);
    },

    /**
     *
     * @private
     */
    _initialize: function () {
        this.updateFrame();
        if (!this._doNotRequestStatus)
            this.getStatus();
    }
});

