if (!String.prototype.capitalize) {
    /**
     * Capitalize first letter in this string.
     * For example: "hello world" -> "Hello world"
     * http://forwebonly.com/capitalize-the-first-letter-of-a-string-in-javascript-the-fast-way/
     * @return {String}
     */
    String.prototype.capitalize = function () {
        if (this.length <= 0)
            return this;
        var c = this.charAt(0);
        var cu = c.toUpperCase();
        if (c === cu)
            return this;
        return this.length === 1 ? cu : cu + this.substring(1);
    };
}

var QnUtil = {};

////////////////////////
// INHERITANCE start ...

/**
 * @param {Function} subclass
 * @param {Function} superclass
 * Does inheritance
 */
QnUtil.inherits = function (subclass, superclass) {
    if (typeof superclass !== 'function') {
        var message = "QnUtil.inherits : superclass argument [" + superclass + "] must be a function (subclass: [" + subclass + "']";
        throw new Error(message);
    }

    subclass.prototype = Object.create(superclass.prototype);
    subclass.prototype.constructor = subclass;
    subclass.superclass = superclass.prototype;
};

/**
 * Checks for inheritance (or same function).
 * Examples :
 * 1. QnUtil.isInherits(TabBase, ViewBase) will return TRUE.
 * 2. QnUtil.isInherits(ViewBase, ViewBase)) will return TRUE.
 * 3. QnUtil.isInherits(TabBase, FrameMain)) will return FALSE.
 * @param {Function} subclass
 * @param {Function} superclass
 * @return {boolean}
 */
QnUtil.isInherits = function (subclass, superclass) {
    if (typeof subclass !== 'function' || typeof superclass !== 'function') {
        var message = "QnUtil.isInherits : superclass argument [" + superclass + "] must be a function (subclass: [" + subclass + "']";
        throw new Error(message);
    }

    return subclass === superclass || subclass.superclass === superclass.prototype;
};

/**
 * Does extending
 * @param {Object} constructorOrConstructorPrototype
 * @param {Object} props
 */
QnUtil._extendImpl = function (constructorOrConstructorPrototype, props) {
    var prototype;
    if (typeof constructorOrConstructorPrototype === 'function') {
        prototype = constructorOrConstructorPrototype.prototype;
    } else {
        prototype = constructorOrConstructorPrototype
    }

    // simple property assignments
    // this is the same Object.assign(constructor.prototype, props);
    for (var key in props) {
        // protect if props.hasOwnProperty() is shadowed
        if (Object.prototype.hasOwnProperty.call(props, key)) {
            prototype[key] = props[key];
        }
    }

    // IE doesn't recognize custom toStrings in for..in
    if (Object.prototype.hasOwnProperty.call(props, "toString") && props["toString"] !== prototype["toString"]) {
        prototype.toString = props.toString;
    }
    return constructorOrConstructorPrototype; // Object
};

/**
 * Does extending
 * @param {Object} constructorOrConstructorPrototype
 * @param {Object...} varArgs
 */
QnUtil.extend = function (constructorOrConstructorPrototype, varArgs) {
    for (var i = 1, l = arguments.length; i < l; i++) {
        var nextProps = arguments[i];
        if (nextProps !== null) {
            QnUtil._extendImpl(constructorOrConstructorPrototype, nextProps);
        }
    }
    return constructorOrConstructorPrototype; // Object
};

// INHERITANCE end ...
////////////////////////

QnUtil.isDefined = function (obj) {
    return null !== obj && undefined !== obj;
};

QnUtil.isStr = function (obj) {
    return obj instanceof String || typeof obj === "string";
};

QnUtil.isBool = function (obj) {
    return obj instanceof Boolean || typeof obj === "boolean";
};

QnUtil.isNumber = function (obj) {
    return obj instanceof Number || typeof obj === "number";
};

// Use JavScript's native support Array.isArray - it's just ES5
QnUtil.isArray = Array.isArray ? Array.isArray : function (wh) {
    if (!wh) {
        return false;
    }
    return (wh instanceof Array || typeof wh === "array");
};

QnUtil.isObject = function (wh) {
    if (!wh) {
        return false;
    }
    return typeof wh === "object";
};

// Protocol utils

/**
 * Check whether given value is with given type/class.
 *
 * @param {Object} value value to be checked
 * @param {String} tag name/key/tag to be used for log
 * @param {String} type expected type like "boolean", "string", etc.
 * @param {Boolean} [noErrorIfInvalid=false] true to not report exception, but to return value.
 * @return {Boolean} true if valid, false if not. Can be returned only if noErrorIfInvalid=true
 * @private
 */
QnUtil.checkType = function (value, tag, type, noErrorIfInvalid) {
    var isValid;
    if (type === "array") {
        isValid = QnUtil.isArray(value);
    } else {
        isValid = (typeof value) === type;
    }
    if (!isValid) {
        var msg = "Invalid type '" + (typeof value) + "' with tag '" +
            tag + "' expects='" + type + "'";
        if (!noErrorIfInvalid)
            throw new Error("Invalid type '" + (typeof value) + "' with tag '" +
                tag + "' expects='" + type + "'");
        else
            logWarn(msg);
    }
    return isValid;
};

/**
 * Get from given JSON root node the specified property.
 * If isOpt=true, and property does not exists or if invalid, returns valueDef.
 * Otherwise report an exception.
 *
 * @param {Object} root JSON node. Must be valid
 * @param {String} tag tag/key of property to be get
 * @param {String} type expected type like "boolean", "string", etc.
 * @param {Boolean} isOpt true if optional
 * @param {String|Boolean|Number|null} valueDef default value to be used if extracted is missing
 * or invalid
 * @returns {String|Boolean|Number|null} same as expected type. Can be null/undefined if valueDef is such
 * @private
 */
QnUtil._getObjType = function (root, tag, type, isOpt, valueDef) {
    isOpt = !!isOpt;
    if (QnUtil.isDefined(valueDef))
        QnUtil.checkType(valueDef, tag, type);

    var value = root[tag];
    if (QnUtil.isDefined(value)) {
        if (QnUtil.checkType(value, tag, type, isOpt)) {
            return value;
        }
    } else {
        if (!isOpt)
            throw new Error("No value wit with tag '" + tag + "'");
    }
    return valueDef;
};

/**
 * Get from given JSON root node the specified property with type Boolean if exists.
 * If not exists or if invalid, returns valueDef.
 *
 * @param {Object} root JSON node. Must be valid
 * @param {String} tag tag/key of property to be get
 * @param {Boolean|null} [valueDef] can be null/undefined, otherwise must be Boolean
 * @return {Boolean} can be null/undefined if valueDef=null/undefined
 */
QnUtil.getBoolOpt = function (root, tag, valueDef) {
    return QnUtil._getObjType(root, tag, "boolean", true, valueDef);
};

/**
 * Get from given JSON root node the specified property with type String if exists.
 * If not exists or if invalid, returns valueDef.
 *
 * @param {Object} root JSON node. Must be valid
 * @param {String} tag tag/key of property to be get
 * @param {String|null} [valueDef] can be null/undefined, otherwise must be String.
 * @return {String} can be null/undefined if valueDef=null/undefined
 */
QnUtil.getStrOpt = function (root, tag, valueDef) {
    return QnUtil._getObjType(root, tag, "string", true, valueDef);
};

/**
 * Get from given JSON root node the specified property with type Number if exists.
 * If not exists or if invalid, returns valueDef.
 *
 * @param {Object} root
 * @param {String} tag
 * @param {Number} valueDef default value to be used if there is no such tag
 * @return {Number} can be null/undefined if valueDef=null/undefined. Otherwise a valid number.
 */
QnUtil.getNumberOpt = function (root, tag, valueDef) {
    var value = QnUtil._getObjType(root, tag, "number", true, valueDef);
    //QnUtil._checkNumber(value, min, max, tag);
    return value;
};

/**
 * Get from given JSON root node the specified property with type String.
 * If not exists or if invalid, report an exception.
 *
 * @param {Object} root JSON node. Must be valid
 * @param {String} tag tag/key of property to be get
 * @return {String} always valid
 */
QnUtil.getStr = function (root, tag) {
    return QnUtil._getObjType(root, tag, "string", false, null);
};

/**
 * Get from given JSON root node the specified property with type Number.
 * If not exists or if invalid, report an exception.
 *
 * @param {Object} root JSON node. Must be valid
 * @param {String} tag tag/key of property to be get
 * @return {Number} always valid
 */
QnUtil.getNumber = function (root, tag) {
    return QnUtil._getObjType(root, tag, "number", false, null);
};
